function [g,a,b,pol,zer,z,f,w,errvec] = aaanonper1(F,Z,P,tol,mmax)
% 
% based on modification of the AAA algorithm in Y. Nakatsukasa, O.Sete, and 
% L.N.Trefethen, The AAA algorithm for rational approximation,
% SIAM J, Sci. Comput. 40(3) (2018), A1494-A1522.
% 
%
%
% Input: F = vector of given Fourier coefficients
%        Z = vector of indices of given Fourier coefficients
%        P = fixed period used for the computation of Fourier coefficients
%        tol = relative tolerance tol, set to 1e-13 if omitted
%        mmax: max type is (mmax-1,mmax-1), set to 100 if omitted
%         
% Output: g vector of coefficients in the generalized Fourier sum 
%         a vector of frequencies in the generalized Fourier sum
%         b vector of shifts in the generalized Fourier sum 
%         pol,zer = vectors of poles, zeros of the rational function
%         (non-periodic part) 
%         z,f,w = vectors of support pts, interpolation values, weights to
%         define the rational function
%         errvec = vector of errors at each step

M = length(Z); % number of given Fourier coefficients
if nargin<4, tol = 1e-13; end % default relative tol 1e-13
if nargin<5, mmax = 100; end % default max type (99,99)
Z=Z(:);F=F(:); % work with column vectors
F = complex(real(F),imag(F)./Z); % change to modified coefficients
FF=isnan(imag(F)); % only necessaray if Z contains index 0
k=find(FF==1,1); % remove imaginary part of c_0(f)
F(k)=real(F(k));
Z = Z.*Z;              % change to squared indices
SF = spdiags(F,0,M,M); % diagonal matrix with values of F
J = 1:M; z = []; f = []; J=J(:); %J1=[]; z1=[];%J1 collects periodic part indices
C = []; errvec = []; R=zeros(M,1); R=R(:); % initializations
% find initial rational function
% find the two coefficients with largest absolute value
[~,k1] = max(abs(F-R)); j1=find(J==k1);
R(j1)=F(j1);
[~,k2] = max(abs(F-R)); j2=find(J==k2);
z=[z;Z(j1);Z(j2)];  % update index set for interpolation
f=[f;F(j1);F(j2)]; % pick the two largest Fourier coefficients
J(J==j1)=[]; J(J==j2)=[];             % update index set
C=[C 1./(Z-Z(j1))]; C=[C 1./(Z-Z(j2))]; % update Cauchy matrix
w=zeros(2,1); % initialize weights vector
w(1)=-F(j2)/sqrt(abs(F(j1))^2+abs(F(j2))^2);
w(2)=F(j1)/sqrt(abs(F(j1))^2+abs(F(j2))^2);
N = C*(w.*f);
D = C*w; % numerator and denominator
k=find(abs(D)<tol);
R = N./D; % initial rational approximation
err = norm(F(J)-R(J),inf);

errvec = [errvec; err]; % max error at sample points
% if err <= tol*norm(F,inf), break, end % stop if converged
for m = 3:mmax   % main loop
[~,j] = max(abs(F(J)-R(J)));  % select next support point
z = [z; Z(J(j))];
f = [f; F(J(j))]; % update support points, data values
C = [C 1./(Z-Z(J(j)))];  % next column of Cauchy matrix
J(j) = []; % update index vector
Sf = diag(f); % right scaling matrix
A=SF*C-C*Sf; % Loewner matrix
A=A(J,:);
[~,G,V] = svd(A); % SVD
w0 = V(:,m); % last singular vector of A
w1=V(:,m-1); % second last singular vector of A
w=(w1'*f).*w0 - (w0'*f).*w1; % weight vector
w=w./norm(w); % normalization of weigt vector
N = C*(w.*f); D = C*w; % numerator and denominator
R = F;
R(J) = N(J)./D(J); % rational approximation
err = norm(F(J)-R(J),inf);
errvec = [errvec; err]; % max error at sample points
if err <= tol*norm(F,inf), break, end % stop if converged
end

j2=find(abs(w)<=1e-7); %find possible zero components in w
w(j2)=[];              %remove zero components in w
z(j2)=[];              % update corresponding interpolation points
f(j2)=[];              % update coressponding values

[AB,pol,zer] = prz(z,f,w); % poles and zeros and partial decomposition
[g,a,b] = trans(pol,P,AB,F,Z,z,w,f);


% computation of the periodic part in the postprocessing step
 Z1 = setdiff(Z,z,'stable');
 F1= setdiff(F,f,'stable');
 CC = 1./bsxfun(@minus,Z1,z.'); %Cauchy matrix
 r = (CC*(w.*f))./(CC*w); %compute rational function at Z
 f2=F1- r; %compute the difference between Fourier coefficientvector and rational function  
 ii = find(abs(f2)>0.000001); % find indices belonging to periodic part
 if ~isempty(ii)
     r=length(ii);
     for k=1:r
 n=find(Z==Z1(ii(k)));
 a=[a;sqrt(Z(n))./P];
     end
 f2(ii)=complex(real(f2(ii)),imag(f2(ii)).*sqrt(Z1(ii)));
 g=[g;abs(f2(ii)).*2];
 b=[b;-acot(real(f2(ii))./imag(f2(ii)))];
   
 end
 z=sqrt(z); %change back to indices vector
 
function [AB,pol,zer] = prz(z,f,w) 
% compute poles pol, zeros zer, of rational function
% AB complex nominator vector in and partial fraction decomposition
m = length(w); 
B = eye(m+1);
B(1,1) = 0;
E = [0 w.'; ones(m,1) diag(z)];
pol = eig(E,B); pol = pol(~isinf(pol)); %  poles
E = [0 (w.*f).'; ones(m,1) diag(z)];
zer = eig(E,B); zer = zer(~isinf(zer)); %  zeros
pol=real(pol);
CC=(-1)./bsxfun(@minus,z, pol.');
AB=linsolve(CC,f);


% function to compute the parameter vectors
function [g,a,b] = trans(pol,P,AB,F,Z,z,w,f) 
j1=find(abs(AB)<=1e-7);  % find the zero components of numerator
AB(j1)=[];
pol(j1)=[];
K=length(pol);
a=sqrt(pol);
ss=sin(pi.*a);
g=sqrt(real(AB).^2+pol.*imag(AB).^2);
g=(abs(g).*pi)./abs(a.*ss);   
b=-real(AB)./imag(AB)./a;
 for j=1:K
 if pol(j)>0 
    b(j)=mod(acot(b(j))-a(j)*pi,pi);
 if sign(sin(b(j)+a(j)*pi))~=sign(-imag(AB(j))*ss(j)) 
     b(j)=b(j)+pi;
 end
 else %only for acosh case
     sii=sign(imag(AB(j)));
     b(j)=acosh(abs(real(AB(j))/g(j))).*sign(imag(AB(j)))-abs(a(j)*pi);
     b(j) = 1i*b(j);
 end 
 end
 a=a./P;
 
 

    






